import os, subprocess, shutil, datetime
import numpy as np

from pkg_resources import resource_filename
from ptsnet.simulation.constants import TACC_FILE_TEMPLATE

def run_shell(command):
    subprocess.run(command.split(' '))

def get_root_path():
    rpath = resource_filename(__name__, '')
    token = 'ptsnet'
    idx = rpath.rfind(token)
    return rpath[:idx+len(token)]

def get_examples_path():
    return os.path.join(get_root_path(), 'examples')

def get_example_path(example_name):
    ename = example_name
    if not example_name.lower().endswith('.inp'):
        ename = ename.upper()
        ename += '.inp'
    return os.path.join(get_examples_path(), ename)

def walk(folder_structure, root_path):
    paths = []

    if type(folder_structure) == dict:
        for ff in folder_structure:
            paths.extend(walk(folder_structure[ff], os.path.join(root_path, ff)))
    elif type(folder_structure) in (list, tuple):
        for ff in folder_structure:
            paths.extend(walk(ff, root_path))
    elif type(folder_structure) == int:
        new_root_path = os.path.join(root_path, str(folder_structure))
        paths.append(new_root_path)

    return paths

def create_tacc_job(
    fpath,
    job_name,
    num_processors,
    allocation,
    run_time,
    processors_per_node = 64,
    queue = 'normal',
    file_args = ''):

    jobs_path = os.path.join(get_temp_folder(), "jobs")
    os.makedirs(jobs_path, exist_ok=True)
    job_path = os.path.join(jobs_path, f'{job_name}.sh')

    job_content = TACC_FILE_TEMPLATE.format(
        job_name = job_name,
        queue = queue,
        num_nodes = int(np.ceil(num_processors/processors_per_node)),
        num_processors = num_processors,
        run_time = str(datetime.timedelta(seconds=run_time*60)),
        allocation = allocation
    )
    job_content += \
        "ml python3\n" + \
        "ml phdf5/1.8.16\n" + \
        f"ibrun python3 {fpath} {file_args}\n"

    with open(job_path, "w") as f:
        f.write(job_content)

def submit_tacc_jobs():
    submit_path = os.path.join(get_temp_folder(), 'submit_jobs.sh')
    jobs_path = os.path.join(get_temp_folder(), "jobs")
    fcontent = \
        "#!/bin/bash\n" + \
        f"for f in {jobs_path}/*.sh;\n" + \
        "    do sbatch ${f};\n" + \
        "done\n"
    with open(submit_path, "w") as f:
        f.write(fcontent)
    os.system(f'bash {submit_path}')

def export_time_series(times, data, path):
    '''
        data is a dictionary
    '''
    labels = ['Time']; results = []

    for label in data:
        labels.append(label)
        results.append(data[label])
    header = ','.join(labels)
    np.savetxt(path, list(zip(times, *results)), delimiter=',', header=header, comments='')


import shutil
import os
import pickle
import pathlib
import uuid

from datetime import datetime
from ptsnet.utils.io import get_root_path, walk
from pkg_resources import resource_filename
from ptsnet.results.storage import StorageManager
from ptsnet.utils.data import is_array

def get_num_tmp_workspaces():
    return len(list_workspaces())

def get_count_of_tmp_workspaces():
    return os.path.join(os.getcwd(), 'workspaces', 'count.pkl')

def new_workspace_name(is_root = True):
    if is_root:
        if not os.path.exists(get_count_of_tmp_workspaces()):
            count = 0
        else:
            with open(get_count_of_tmp_workspaces(), 'rb') as f:
                count = pickle.load(f)
        count += 1
        with open(get_count_of_tmp_workspaces(), 'wb') as f:
            pickle.dump(count, f)
        return f'W{count}'
    return None

def new_uuid_workspace_name(size=1):
    wnames = []
    for i in range(size):
        wname = uuid.uuid4().hex[:8]
        while os.path.isdir(os.path.join(os.getcwd(), 'workspaces', wname)):
            wname = uuid.uuid4().hex[:8]
        wnames.append(f"W{wname}")
    return wnames

def create_workspaces_folder(root=False):
    ws_path = os.path.join(os.getcwd(), 'workspaces')
    if root:
        if not os.path.exists(ws_path):
            os.mkdir(ws_path)
    return ws_path

def get_workspaces():
    return os.listdir(os.path.join(os.getcwd(), 'workspaces'))

def get_tmp_folder(root=False):
    return os.path.join(create_workspaces_folder(root), 'tmp')

def create_temp_folder(root=False):
    tmpdir = os.path.join(create_workspaces_folder(root), 'tmp')
    if root:
        if os.path.exists(tmpdir): shutil.rmtree(tmpdir)
        os.makedirs(tmpdir)
    return tmpdir

def list_workspaces():
    wps = [d for d in os.listdir(os.path.join(os.getcwd(), 'workspaces')) if '.' not in d].sort()
    return wps

def print_workspaces(full_path = False):
    txt = {}
    for d in list_workspaces():
        abs_path = os.path.join(os.getcwd(), 'workspaces', d)
        with open(os.path.join(abs_path, 'settings.pkl'), 'rb') as f:
            s = pickle.load(f)
        with open(os.path.join(abs_path, 'fname.pkl'), 'rb') as f:
            fname = pickle.load(f)
            if full_path:
                fname = os.path.basename(fname)
        txt[d] = [
            fname,
            f"T = {s['duration']}, t = {s['time_step']}, N = {s['num_points']}, n = {s['num_processors']}",
        ]
    print('\n')
    for i, d in enumerate(txt):
        print(f'  ({i})', txt[d][0])
        print('    ' + txt[d][1])
        print('\n')

def delete_all_workspaces():
    confirmation = None
    while confirmation is None:
        confirmation = input('Are you sure that you want to delete all the workspaces? (yes/no): ')
        if not confirmation in ('yes', 'no'):
            confirmation = None
    if confirmation == 'no':
        return
    ROOT = get_root_path()
    for d in os.listdir(os.path.join(os.getcwd(), 'workspaces')):
        abs_path = os.path.join(os.getcwd(), 'workspaces', d)
        if os.path.isdir(abs_path):
            shutil.rmtree(abs_path)
        else:
            os.remove(abs_path)

def delete_workspace(workspace_name, full_path=True, verbose=True):
    confirmation = None

    ROOT = get_root_path()
    wpl = list_workspaces()
    wnamelist = workspace_name

    txt = {}

    wnames = workspace_name
    if not is_array(workspace_name):
        wnames = [workspace_name]
        apaths = [os.path.join(os.getcwd(), 'workspaces', workspace_name)]
    else:
        apaths = \
            [os.path.join(
                os.getcwd(),
                'workspaces',
                wp_name) for wp_name in workspace_name]

    for i, apath in enumerate(apaths):
        f = pathlib.Path(apath)
        st = f.stat().st_mtime
        with open(os.path.join(apath, 'settings.pkl'), 'rb') as f:
            s = pickle.load(f)
        with open(os.path.join(apath, 'fname.pkl'), 'rb') as f:
            fname = pickle.load(f)
        if not full_path:
            fname = os.path.basename(fname)
        txt = [
            f'Last modification on: {datetime.fromtimestamp(st)}',
            fname,
            f"T = {s['duration']}, t = {s['time_step']}, N = {s['num_points']}, n = {s['num_processors']}",
        ]

        if verbose:
            print('\n')
            print(f'  ({wnames[i]}) ' + txt[0])
            print('      ' + txt[1])
            print('      ' + txt[2])
            print('\n')

            confirmation = None
            while confirmation is None:
                confirmation = input('Are you sure that you want to delete this workspace ? (yes/no): ')
                if not confirmation in ('yes', 'no'):
                    confirmation = None
        else:
            confirmation = 'yes'

        if confirmation == 'no': continue

        if os.path.isdir(apath):
            shutil.rmtree(apath)
        else:
            os.remove(apath)
